"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const arangojs_1 = require("arangojs");
const arangodb_query_1 = require("./arangodb-query");
const sql_utils_1 = require("./sql-utils");
const DbTypeToGenericType = {
    number: 'double',
    string: 'text',
    bool: 'boolean'
};
const sortByKeys = (unordered) => {
    const ordered = {};
    Object.keys(unordered).sort().forEach((key) => {
        ordered[key] = unordered[key];
    });
    return ordered;
};
class ArangoDbDriver {
    constructor(config = {}) {
        this.loadPreAggregationIntoTable = async (preAggregationTableName, loadSql, params, options) => {
            throw new Error('Method not implemented.');
        };
        this.tableColumnTypes = async (table) => {
            const columns = [];
            // TODO: can optimize by schema registry or swagger json schema
            const attrMap = await this.aggrAttrs(table);
            const attrNames = Object.keys(attrMap);
            for (const attrName of attrNames) {
                const attrType = attrMap[attrName];
                if (this.toGenericType(attrType)) {
                    columns.push({ name: attrName, type: attrType, attributes: [] });
                }
            }
            return columns.sort();
        };
        this.getTablesQuery = async (schemaName) => {
            const collections = await this.client.collections();
            return collections.map((col) => ({ table_name: col.name }));
        };
        this.dropTable = (tableName, options) => {
            throw new Error('Method not implemented.');
        };
        this.downloadQueryResults = async (query, values, options) => {
            throw new Error('Method not implemented.');
        };
        this.downloadTable = async (table, options) => {
            throw new Error('Method not implemented.');
        };
        const auth = {
            username: process.env.CUBEJS_DB_USER,
            password: process.env.CUBEJS_DB_PASS,
        };
        this.config = {
            url: process.env.CUBEJS_DB_URL,
            databaseName: process.env.CUBEJS_DB_NAME,
            auth,
            ...config
        };
        this.client = new arangojs_1.Database({
            url: this.config.url,
            databaseName: this.config.databaseName,
            auth: this.config.auth,
            // ssl: this.config.ssl
        });
    }
    /**
     * Returns default concurrency value.
     */
    static getDefaultConcurrency() {
        return 2;
    }
    static driverEnvVariables() {
        return [
            'CUBEJS_DB_URL'
        ];
    }
    static dialectClass() {
        return arangodb_query_1.ArangoDbQuery;
    }
    async testConnection() {
        const cursor = await this.client.query({
            query: 'RETURN @value',
            bindVars: { value: Date.now() }
        });
        return await cursor.next();
    }
    async release() {
        await this.client.close();
    }
    readOnly() {
        // ArangoDb don't support table creation
        return true;
    }
    async tablesSchema() {
        const result = {};
        const collections = await this.client.collections();
        const schemaName = this.config.databaseName;
        let schema = (result[schemaName] || {});
        for (const collection of collections) {
            const collectionMeta = await collection.get();
            if (collectionMeta.type !== arangojs_1.CollectionType.DOCUMENT_COLLECTION)
                continue;
            schema[collection.name] = await this.tableColumnTypes(collection.name);
        }
        schema = sortByKeys(schema);
        result[schemaName] = schema;
        return sortByKeys(result);
    }
    async createSchemaIfNotExists(schemaName) {
        throw new Error('Method not implemented.');
    }
    async uploadTableWithIndexes(table, columns, tableData, indexesSql, uniqueKeyColumns, queryTracingObj) {
        throw new Error('Method not implemented.');
    }
    async query(query, params, options) {
        console.log(query, params, options);
        const aqlQuery = (0, sql_utils_1.sql2aql)(query);
        const cursor = await this.client.query(aqlQuery);
        const result = cursor.all();
        await cursor.kill();
        return result;
    }
    // public stream?: (table: string, values: unknown[], options: StreamOptions) => Promise<StreamTableData>;
    // public unload?: (table: string, options: UnloadOptions) => Promise<DownloadTableCSVData>;
    // public isUnloadSupported?: (options: UnloadOptions) => Promise<boolean>;
    nowTimestamp() {
        return Date.now();
    }
    // TODO: add to interface too
    quoteIdentifier(identifier) {
        return `"${identifier}"`;
    }
    async aggrAttrs(collectionName) {
        const cursor = await this.client.query(`
FOR i IN [1]
  LET attrMaps = (
    FOR doc in ${collectionName}
      LET attributes = (
        FOR name IN ATTRIBUTES(doc, true)
          RETURN {
            name: name,
            type: TYPENAME(doc[name])
          }
      ) 
      RETURN ZIP(attributes[*].name, attributes[*].type)
  )
  RETURN MERGE(attrMaps)`);
        let result = { id: 'string' };
        if (cursor.hasNext) {
            result = {
                ...result,
                ...await cursor.next()
            };
        }
        await cursor.kill();
        return result;
    }
    toGenericType(columnType) {
        return DbTypeToGenericType[columnType.toLowerCase()] || columnType;
    }
}
exports.default = ArangoDbDriver;
//# sourceMappingURL=arangodb-driver.js.map